import { select, drag, geoOrthographic, zoom } from 'd3';
import topojson from 'topojson-client';
import { map } from './map';

const worldAtlasURL =
  'https://unpkg.com/visionscarto-world-atlas@0.1.0/world/110m.json';
const submarineCablesURL =
  'https://raw.githubusercontent.com/curran/www.submarinecablemap.com/master/web/public/api/v3/cable/cable-geo.json';
const enableAnimation = true;

// Sensitivity of the drag interaction
const panningSensitivity = 58;

// White space around the outside.
const margin = 20;

export const main = (container, { state, setState }) => {
  // Measure the size of the container
  const width = container.clientWidth;
  const height = container.clientHeight;

  // Scaffold the top-level SVG element
  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  // Unpack the state
  const {
    worldAtlasData,
    submarineCablesData,
    rotation,
    scale,
    translation,
    initialScale,
  } = state;

  // Initialize the World Atlas data
  if (worldAtlasData === undefined) {
    setState((state) => ({
      ...state,
      worldAtlasData: 'LOADING',
    }));
    fetch(worldAtlasURL)
      .then((response) => response.json())
      .then((topoJSONData) => {
        const worldAtlasData = topojson.feature(
          topoJSONData,
          'countries',
        );
        setState((state) => ({
          ...state,
          worldAtlasData,
        }));
      });
  }

  // Initialize the Submarine Cables data
  if (submarineCablesData === undefined) {
    setState((state) => ({
      ...state,
      submarineCablesData: 'LOADING',
    }));
    fetch(submarineCablesURL)
      .then((response) => response.json())
      .then((submarineCablesData) => {
        // const worldAtlasData = topojson.feature(
        //   topoJSONData,
        //   'countries',
        // );
        setState((state) => ({
          ...state,
          submarineCablesData,
        }));
      });
  }

  // Wait for the World Atlas data to load
  // before proceeding further.
  if (
    worldAtlasData === undefined ||
    worldAtlasData === 'LOADING' ||
    submarineCablesData === undefined ||
    submarineCablesData === 'LOADING'
  ) {
    return;
  }

  // Initialize rotation
  if (rotation === undefined) {
    setState((state) => ({
      ...state,
      rotation: [0, -20, 0],
    }));
    return;
  }

  // Initialize scale and translation
  // by fitting to the size of the page
  if (scale === undefined) {
    const fittedProjection = geoOrthographic().fitExtent(
      [
        [margin, margin],
        [width - margin, height - margin],
      ],
      { type: 'Sphere' },
    );
    const initialScale = fittedProjection.scale();
    const initialTranslation = fittedProjection.translate();
    setState((state) => ({
      ...state,
      initialScale,
      scale: initialScale,
      translation: initialTranslation,
    }));
    return;
  }

  // Render the map
  svg.call(map, {
    worldAtlasData,
    submarineCablesData,
    rotation,
    scale,
    translation,
  });

  // Set up the drag behavior
  svg.call(
    drag().on('drag', (event) => {
      setState((state) => {
        const k = panningSensitivity / state.scale;
        return {
          ...state,
          rotation: [
            state.rotation[0] + event.dx * k,
            state.rotation[1] - event.dy * k,
            state.rotation[2],
          ],
        };
      });
    }),
  );

  // Set up the zoom behavior
  svg.call(
    zoom().on('zoom', ({ transform: { k } }) => {
      setState((state) => ({
        ...state,
        scale: state.initialScale * k,
      }));
    }),
  );
};
